@@ -1,10 +1,14 @@
 use clap::{App, Arg};
 use regex::Regex;
 
-fn integer_value(v: String) -> Result<(), String> {
+fn port_value(v: String) -> Result<(), String> {
   v.parse::<u16>().map(|_| ()).map_err(|e| format!("'{}' is not a valid port value: {}", v, e) )
 }
 
+fn integer_value(v: String) -> Result<(), String> {
+  v.parse::<u64>().map(|_| ()).map_err(|e| format!("'{}' is not a valid integer value: {}", v, e) )
+}
+
 pub(crate) fn setup_app<'a, 'b>(program: String, version: &'b str) -> App<'a, 'b> {
   App::new(program)
     .version(version)
@@ -69,7 +73,7 @@ pub(crate) fn setup_app<'a, 'b>(program: String, version: &'b str) -> App<'a, 'b
       .takes_value(true)
       .use_delimiter(false)
       .help("Provider port (defaults to protocol default 80/443)")
-      .validator(integer_value))
+      .validator(port_value))
     .arg(Arg::with_name("scheme")
       .long("scheme")
       .takes_value(true)
@@ -229,16 +233,31 @@ pub(crate) fn setup_app<'a, 'b>(program: String, version: &'b str) -> App<'a, 'b
       .empty_values(false)
       .validator(integer_value)
       .help("Sets the HTTP request timeout in milliseconds for requests to the target API and for state change requests."))
-    }
+    .arg(Arg::with_name("json-file")
+      .short("j")
+      .long("json")
+      .takes_value(true)
+      .use_delimiter(false)
+      .multiple(false)
+      .number_of_values(1)
+      .empty_values(false)
+      .help("Generate a JSON report of the verification"))
+}
 
 #[cfg(test)]
 mod test {
-  use super::integer_value;
+  use super::{integer_value, port_value};
   use expectest::prelude::*;
 
+  #[test]
+  fn validates_port_value() {
+    expect!(port_value("1234".to_string())).to(be_ok());
+    expect!(port_value("1234x".to_string())).to(be_err());
+  }
+
   #[test]
   fn validates_integer_value() {
-    expect!(integer_value("1234".to_string())).to(be_ok());
+    expect!(integer_value("3000000".to_string())).to(be_ok());
     expect!(integer_value("1234x".to_string())).to(be_err());
   }
 }
